@@ -9,7 +9,6 @@
  *****************************************************************************/
 package org.picocontainer;
 
-import java.util.Arrays;
 import java.util.Map;
 import java.util.Properties;
 
@@ -19,6 +18,7 @@ import java.util.Properties;
  * of Characteristics.
  * 
  * @author Paul Hammant
+ * @see org.picocontainer.ComponentAdapter
  * @see org.picocontainer.Behavior
  */
 @SuppressWarnings("serial")
@@ -36,6 +36,8 @@ public final class Characteristics {
     private static final String _PROPERTY_APPLYING = "property-applying";
     private static final String _AUTOMATIC = "automatic";
     private static final String _USE_NAMES = "use-parameter-names";    
+    private static final String _ENABLE_CIRCULAR = "enable-circular";
+    private static final String _GUARD = "guard";
 
     /**
      * Since properties use strings, we supply String constants for Boolean conditions.
@@ -129,7 +131,8 @@ public final class Characteristics {
      * @see #HIDE_IMPL for more information.
      */
     public static final Properties NO_HIDE_IMPL = immutable(_HIDE_IMPL, FALSE);
-    
+
+    public static final Properties ENABLE_CIRCULAR = immutable(_ENABLE_CIRCULAR, TRUE);
     
     public static final Properties NONE = immutable(_NONE, "");
 
@@ -138,12 +141,23 @@ public final class Characteristics {
      * after the object is created based.
      */
     public static final Properties PROPERTY_APPLYING = immutable(_PROPERTY_APPLYING, TRUE);
+    
+    /**
+     * Turns off bean-setting property applications.
+     * @see org.picocontainer.behaviors.PropertyApplying
+     */
+    public static final Properties NO_PROPERTY_APPLYING = immutable(_PROPERTY_APPLYING, FALSE);
 
     public static final Properties AUTOMATIC = immutable(_AUTOMATIC, TRUE);
 
     public static final Properties USE_NAMES = immutable(_USE_NAMES, TRUE);
 
-    
+    public static final Properties GUARD = immutable(_GUARD, "guard");
+
+    public static final Properties GUARD(String with) {
+        return immutable(_GUARD, with);
+    };
+
     /**
      * Transforms a single name value pair unto a <em>read only</em> {@linkplain java.util.Properties}
      * instance.
@@ -174,22 +188,34 @@ public final class Characteristics {
             sealed = true;
         }
         
-        
+        /**
+         * Read Only Object:  will throw UnsupportedOperationException.
+         */
         @Override
         public Object remove(Object o) {
             throw new UnsupportedOperationException("immutable properties are read only");
         }
 
+        /**
+         * Read Only Object:  will throw UnsupportedOperationException.
+         */
         @Override
         public synchronized Object setProperty(String string, String string1) {
             throw new UnsupportedOperationException("immutable properties are read only");
         }
 
+        /**
+         * Read Only Object:  will throw UnsupportedOperationException.
+         */
 		@Override
 		public synchronized void clear() {
             throw new UnsupportedOperationException("immutable properties are read only");
 		}
 
+		/**
+		 * Once object is constructed, this will throw UnsupportedOperationException because
+		 * this class is a read only wrapper.
+		 */
 		@Override
 		public synchronized Object put(Object key, Object value) {
 			if (!sealed) {
@@ -201,6 +227,9 @@ public final class Characteristics {
             throw new UnsupportedOperationException("immutable properties are read only");
 		}
 
+        /**
+         * Read Only Object:  will throw UnsupportedOperationException.
+         */
 		@Override
 		public synchronized void putAll(Map<? extends Object, ? extends Object> t) {
             throw new UnsupportedOperationException("immutable properties are read only");
