@@ -25,6 +25,7 @@ import java.util.Properties;
 import java.util.Set;
 
 import org.picocontainer.adapters.InstanceAdapter;
+import org.picocontainer.adapters.AbstractAdapter;
 import org.picocontainer.behaviors.AbstractBehaviorFactory;
 import org.picocontainer.behaviors.AdaptingBehavior;
 import org.picocontainer.behaviors.Cached;
@@ -41,6 +42,7 @@ import org.picocontainer.lifecycle.DefaultLifecycleState;
 import org.picocontainer.lifecycle.LifecycleState;
 import org.picocontainer.lifecycle.StartableLifecycleStrategy;
 import org.picocontainer.monitors.NullComponentMonitor;
+import org.picocontainer.parameters.DefaultConstructorParameter;
 
 /**
  * <p/>
@@ -106,18 +108,17 @@ public class DefaultPicoContainer implements MutablePicoContainer, ComponentMoni
      * Lifecycle strategy instance.
      */
     protected final LifecycleStrategy lifecycleStrategy;
-    
-    private final Properties componentProperties = new Properties();
+
+    /**
+     * Properties set at the container level, that will affect subsequent components added.
+     */
+    private final Properties containerProperties = new Properties();
     
     /**
      * Component monitor instance.  Receives event callbacks.
      */
     protected ComponentMonitor componentMonitor;
 
-    /** List collecting the CAs which have been successfully started */
-    private final List<WeakReference<ComponentAdapter<?>>> startedComponentAdapters = new ArrayList<WeakReference<ComponentAdapter<?>>>();
-
-
     /**
      * Map used for looking up component adapters by their key.
      */
@@ -271,9 +272,39 @@ public class DefaultPicoContainer implements MutablePicoContainer, ComponentMoni
         if (adapter == null && parent != null) {
             adapter = getParent().getComponentAdapter(componentKey);
         }
+        if (adapter == null) {
+            Object inst = componentMonitor.noComponentFound(this, componentKey);
+            if (inst != null) {
+                adapter = new LateInstance(componentKey, inst);
+            }
+        }
         return adapter;
     }
 
+    public static class LateInstance extends AbstractAdapter {
+        private final Object instance;
+        private LateInstance(Object componentKey, Object instance) {
+            super(componentKey, instance.getClass());
+            this.instance = instance;
+        }
+
+        public Object getComponentInstance() {
+            return instance;
+        }
+
+        public Object getComponentInstance(PicoContainer container, Type into) throws PicoCompositionException {
+            return instance;
+        }
+
+        public void verify(PicoContainer container) throws PicoCompositionException {
+        }
+
+        public String getDescriptor() {
+            return "LateInstance";
+        }
+    }
+
+
     /** {@inheritDoc} **/
     public <T> ComponentAdapter<T> getComponentAdapter(final Class<T> componentType, final NameBinding componentNameBinding) {
         return getComponentAdapter(componentType, componentNameBinding, null);
@@ -360,7 +391,7 @@ public class DefaultPicoContainer implements MutablePicoContainer, ComponentMoni
      * passed to the constructor of this container.
      */
     public MutablePicoContainer addAdapter(final ComponentAdapter<?> componentAdapter) {
-        return addAdapter(componentAdapter,  this.componentProperties);
+        return addAdapter(componentAdapter,  this.containerProperties);
     }
 
     /** {@inheritDoc} **/
@@ -396,7 +427,7 @@ public class DefaultPicoContainer implements MutablePicoContainer, ComponentMoni
      * The returned ComponentAdapter will be an {@link org.picocontainer.adapters.InstanceAdapter}.
      */
     public MutablePicoContainer addComponent(final Object implOrInstance) {
-        return addComponent(implOrInstance, this.componentProperties);
+        return addComponent(implOrInstance, this.containerProperties);
     }
 
     private MutablePicoContainer addComponent(final Object implOrInstance, final Properties props) {
@@ -426,16 +457,22 @@ public class DefaultPicoContainer implements MutablePicoContainer, ComponentMoni
     public MutablePicoContainer addComponent(final Object componentKey,
                                              final Object componentImplementationOrInstance,
                                              final Parameter... parameters) {
-        return this.addComponent(componentKey, componentImplementationOrInstance, this.componentProperties, parameters);
+        return this.addComponent(componentKey, componentImplementationOrInstance, this.containerProperties, parameters);
     }
 
     private MutablePicoContainer addComponent(final Object componentKey,
                                              final Object componentImplementationOrInstance,
                                              final Properties properties,
                                              Parameter... parameters) {
-        if (parameters != null && parameters.length == 0 && parameters != Parameter.ZERO) {
+        if (parameters != null && parameters.length == 0) {
             parameters = null; // backwards compatibility!  solve this better later - Paul
         }
+        
+        //New replacement for Parameter.ZERO.
+        if (parameters != null && parameters.length == 1 && DefaultConstructorParameter.INSTANCE.equals(parameters[0])) {
+        	parameters = new Parameter[0];
+        }
+        
         if (componentImplementationOrInstance instanceof Class) {
             Properties tmpProperties = (Properties) properties.clone();
             ComponentAdapter<?> componentAdapter = componentFactory.createComponentAdapter(componentMonitor,
@@ -529,18 +566,14 @@ public class DefaultPicoContainer implements MutablePicoContainer, ComponentMoni
         Object retVal;
         if (annotation != null) {
             final ComponentAdapter<?> componentAdapter = getComponentAdapter((Class<?>)componentKeyOrType, annotation);
-            retVal = componentAdapter == null ? null : getInstance(componentAdapter, null);
+            return componentAdapter == null ? null : getInstance(componentAdapter, null);
         } else if (componentKeyOrType instanceof Class) {
             final ComponentAdapter<?> componentAdapter = getComponentAdapter((Class<?>)componentKeyOrType, (NameBinding) null);
-            retVal = componentAdapter == null ? null : getInstance(componentAdapter, (Class<?>)componentKeyOrType);
+            return componentAdapter == null ? null : getInstance(componentAdapter, (Class<?>)componentKeyOrType);
         } else {
             ComponentAdapter<?> componentAdapter = getComponentAdapter(componentKeyOrType);
-            retVal = componentAdapter == null ? null : getInstance(componentAdapter, null);
+            return componentAdapter == null ? null : getInstance(componentAdapter, null);
         }
-        if (retVal == null) {
-            retVal = componentMonitor.noComponentFound(this, componentKeyOrType);
-        }
-        return retVal;
     }
 
     public <T> T getComponent(final Class<T> componentType) {
@@ -725,7 +758,7 @@ public class DefaultPicoContainer implements MutablePicoContainer, ComponentMoni
     }
 
     public MutablePicoContainer makeChildContainer() {
-        DefaultPicoContainer pc = new DefaultPicoContainer(componentFactory, lifecycleStrategy, this);
+        DefaultPicoContainer pc = new DefaultPicoContainer(componentFactory, lifecycleStrategy, this, componentMonitor);
         addChildContainer(pc);
         return pc;
     }
@@ -737,10 +770,9 @@ public class DefaultPicoContainer implements MutablePicoContainer, ComponentMoni
      * @param child
      */
     private void checkCircularChildDependencies(PicoContainer child) {
-    	final String MESSAGE = "Cannot have circular dependency between parent " 
-			+ this + " and child: " + child;
+    	final String MESSAGE = "Cannot have circular dependency between parent %s and child: %s";
     	if (child == this) {
-    		throw new IllegalArgumentException(MESSAGE);
+    		throw new IllegalArgumentException(String.format(MESSAGE,this,child));
     	}
     	
     	//Todo: Circular Import Dependency on AbstractDelegatingPicoContainer
@@ -749,14 +781,13 @@ public class DefaultPicoContainer implements MutablePicoContainer, ComponentMoni
     		while(delegateChild != null) {
     			PicoContainer delegateInstance = delegateChild.getDelegate();
     			if (this == delegateInstance) {
-					throw new IllegalArgumentException(MESSAGE);
+					throw new IllegalArgumentException(String.format(MESSAGE,this,child));
     			}
     			if (delegateInstance instanceof AbstractDelegatingPicoContainer) {
     				delegateChild = (AbstractDelegatingPicoContainer) delegateInstance;
     			} else {
     				delegateChild = null;
     			}
-    			
     		}
     	}
     	
@@ -796,7 +827,7 @@ public class DefaultPicoContainer implements MutablePicoContainer, ComponentMoni
             Enumeration<String> e = (Enumeration<String>) c.propertyNames();
             while (e.hasMoreElements()) {
                 String s = e.nextElement();
-                componentProperties.setProperty(s,c.getProperty(s));
+                containerProperties.setProperty(s,c.getProperty(s));
             }
         }
         return this;
@@ -808,7 +839,7 @@ public class DefaultPicoContainer implements MutablePicoContainer, ComponentMoni
 
     public void accept(final PicoVisitor visitor) {
     	
-    	//Pico 3 todo, change accept signatures to allow abort at any point in the traversal.
+    	//TODO Pico 3 : change accept signatures to allow abort at any point in the traversal.
         boolean shouldContinue = visitor.visitContainer(this);
         if (!shouldContinue) {
         	return;
@@ -867,9 +898,9 @@ public class DefaultPicoContainer implements MutablePicoContainer, ComponentMoni
     private void startAdapters() {
         Collection<ComponentAdapter<?>> adapters = getComponentAdapters();
         for (ComponentAdapter<?> adapter : adapters) {
-            if (adapter instanceof Behavior) {
-                Behavior<?> behaviorAdapter = (Behavior<?>)adapter;
-                if (behaviorAdapter.componentHasLifecycle()) {
+            if (adapter instanceof ComponentLifecycle) {
+                ComponentLifecycle<?> componentLifecycle = (ComponentLifecycle<?>)adapter;
+                if (componentLifecycle.componentHasLifecycle()) {
                     // create an instance, it will be added to the ordered CA list
                     adapter.getComponentInstance(DefaultPicoContainer.this, ComponentAdapter.NOTHING.class);
                     addOrderedComponentAdapter(adapter);
@@ -877,15 +908,12 @@ public class DefaultPicoContainer implements MutablePicoContainer, ComponentMoni
             }
         }
         adapters = getOrderedComponentAdapters();
-        // clear list of started CAs
-        startedComponentAdapters.clear();
         // clone the adapters
         List<ComponentAdapter<?>> adaptersClone = new ArrayList<ComponentAdapter<?>>(adapters);
         for (final ComponentAdapter<?> adapter : adaptersClone) {
-            if (adapter instanceof Behavior) {
-                Behavior<?> manager = (Behavior<?>)adapter;
-                manager.start(DefaultPicoContainer.this);
-                startedComponentAdapters.add(new WeakReference<ComponentAdapter<?>>(adapter));
+            if (adapter instanceof ComponentLifecycle) {
+                ComponentLifecycle<?> componentLifecycle = (ComponentLifecycle<?>)adapter;
+                componentLifecycle.start(DefaultPicoContainer.this);
             }
         }
     }
@@ -896,15 +924,13 @@ public class DefaultPicoContainer implements MutablePicoContainer, ComponentMoni
      * stop(PicoContainer) method on the ones which are LifecycleManagers
      */
     private void stopAdapters() {
-        for (int i = startedComponentAdapters.size() - 1; 0 <= i; i--) {
-            ComponentAdapter<?> adapter = startedComponentAdapters.get(i).get();
-            if (adapter == null) {
-            	//Weak reference -- may be null
-            	continue;
-            }
-            if (adapter instanceof Behavior) {
-                Behavior<?> manager = (Behavior<?>)adapter;
-                manager.stop(DefaultPicoContainer.this);
+        for (int i = getOrderedComponentAdapters().size() - 1; 0 <= i; i--) {
+            ComponentAdapter<?> adapter = getOrderedComponentAdapters().get(i);
+            if (adapter instanceof ComponentLifecycle) {
+                ComponentLifecycle<?> componentLifecycle = (ComponentLifecycle<?>)adapter;
+                if (componentLifecycle.componentHasLifecycle() && componentLifecycle.isStarted()) {
+                    componentLifecycle.stop(DefaultPicoContainer.this);
+                }
             }
         }
     }
@@ -917,9 +943,9 @@ public class DefaultPicoContainer implements MutablePicoContainer, ComponentMoni
     private void disposeAdapters() {
         for (int i = getOrderedComponentAdapters().size() - 1; 0 <= i; i--) {
             ComponentAdapter<?> adapter = getOrderedComponentAdapters().get(i);
-            if (adapter instanceof Behavior) {
-                Behavior<?>manager = (Behavior<?>)adapter;
-                manager.dispose(DefaultPicoContainer.this);
+            if (adapter instanceof ComponentLifecycle) {
+                ComponentLifecycle<?> componentLifecycle = (ComponentLifecycle<?>)adapter;
+                componentLifecycle.dispose(DefaultPicoContainer.this);
             }
         }
     }
@@ -933,8 +959,6 @@ public class DefaultPicoContainer implements MutablePicoContainer, ComponentMoni
 		return orderedComponentAdapters;
 	}
 
-
-
 	/**
 	 * @return the componentKeyToAdapterCache
 	 */
@@ -964,7 +988,7 @@ public class DefaultPicoContainer implements MutablePicoContainer, ComponentMoni
 
         public AsPropertiesPicoContainer(final Properties... props) {
             super(DefaultPicoContainer.this);
-            properties = (Properties) componentProperties.clone();
+            properties = (Properties) containerProperties.clone();
             for (Properties c : props) {
                 Enumeration<?> e = c.propertyNames();
                 while (e.hasMoreElements()) {
@@ -974,6 +998,11 @@ public class DefaultPicoContainer implements MutablePicoContainer, ComponentMoni
             }
         }
 
+        @Override
+        public MutablePicoContainer as(Properties... properties) {
+            throw new PicoCompositionException("Syntax 'as(FOO).as(BAR)' not allowed, do 'as(FOO, BAR)' instead");
+        }
+
         @Override
 		public MutablePicoContainer makeChildContainer() {
             return getDelegate().makeChildContainer();
@@ -1005,6 +1034,4 @@ public class DefaultPicoContainer implements MutablePicoContainer, ComponentMoni
             return ComponentAdapter.NOTHING.class;
         }
     }
-
-
 }
