@@ -97,7 +97,7 @@ public abstract class AbstractInjector<T> extends AbstractAdapter<T> implements
      * @param parameters the parameter types
      * @return the array with the default parameters.
      */
-    protected Parameter[] createDefaultParameters(final Class[] parameters) {
+    protected Parameter[] createDefaultParameters(final Type[] parameters) {
         Parameter[] componentParameters = new Parameter[parameters.length];
         for (int i = 0; i < parameters.length; i++) {
             componentParameters[i] = ComponentParameter.DEFAULT;
@@ -114,6 +114,10 @@ public abstract class AbstractInjector<T> extends AbstractAdapter<T> implements
 
     public abstract T getComponentInstance(PicoContainer container, Type into) throws PicoCompositionException;
 
+    public Object decorateComponentInstance(PicoContainer container, Type into, T instance) {
+        return null;
+    }
+
     @Override
 	public void accept(final PicoVisitor visitor) {
         super.accept(visitor);
@@ -153,7 +157,8 @@ public abstract class AbstractInjector<T> extends AbstractAdapter<T> implements
      * @throws IllegalAccessException
      * @throws InvocationTargetException
      */
-    protected T newInstance(final Constructor<T> constructor, final Object[] parameters) throws InstantiationException, IllegalAccessException, InvocationTargetException {
+    protected T newInstance(final Constructor<T> constructor, final Object[] parameters)
+            throws InstantiationException, IllegalAccessException, InvocationTargetException {
         return constructor.newInstance(parameters);
     }
     /**
@@ -215,12 +220,25 @@ public abstract class AbstractInjector<T> extends AbstractAdapter<T> implements
         throw new PicoCompositionException(e);
     }
 
-    protected Class<?> box(final Class<?> parameterType) {
-        if (parameterType.isPrimitive()) {
-            if (parameterType == Integer.TYPE) {
+    protected Type box(Type parameterType) {
+        if (parameterType instanceof Class && ((Class) parameterType).isPrimitive()) {
+            String parameterTypeName = ((Class) parameterType).getName();
+            if (parameterTypeName == "int") {
                 return Integer.class;
-            } else if (parameterType == Boolean.TYPE) {
+            } else if (parameterTypeName == "boolean") {
                 return Boolean.class;
+            } else if (parameterTypeName == "long") {
+                return Long.class;
+            } else if (parameterTypeName == "float") {
+                return Float.class;
+            } else if (parameterTypeName == "double") {
+                return Double.class;
+            } else if (parameterTypeName == "char") {
+                return Character.class;
+            } else if (parameterTypeName == "byte") {
+                return Byte.class;
+            } else if (parameterTypeName == "short") {
+                return Short.class;
             }
         }
         return parameterType;
@@ -383,7 +401,7 @@ public abstract class AbstractInjector<T> extends AbstractAdapter<T> implements
 		
 		private final ComponentAdapter<?> instantiatingComponentAdapter;
         private final Set unsatisfiableDependencies;
-        private final Class<?> unsatisfiedDependencyType;
+        private final Type unsatisfiedDependencyType;
         
         /**
          * The original container requesting the instantiation of the component.
@@ -391,7 +409,7 @@ public abstract class AbstractInjector<T> extends AbstractAdapter<T> implements
         private final PicoContainer leafContainer;
 
         public UnsatisfiableDependenciesException(final ComponentAdapter<?> instantiatingComponentAdapter,
-                                                  final Class<?> unsatisfiedDependencyType, final Set unsatisfiableDependencies,
+                                                  final Type unsatisfiedDependencyType, final Set unsatisfiableDependencies,
                                                   final PicoContainer leafContainer) {
             super(instantiatingComponentAdapter.getComponentImplementation().getName() + " has unsatisfied dependency: " + unsatisfiedDependencyType
                     +" among unsatisfiable dependencies: "+unsatisfiableDependencies + " where " + leafContainer
@@ -410,7 +428,7 @@ public abstract class AbstractInjector<T> extends AbstractAdapter<T> implements
             return unsatisfiableDependencies;
         }
 
-        public Class<?> getUnsatisfiedDependencyType() {
+        public Type getUnsatisfiedDependencyType() {
             return unsatisfiedDependencyType;
         }
 
