@@ -11,7 +11,10 @@
 package org.picocontainer.injectors;
 
 import java.lang.reflect.AccessibleObject;
+import java.lang.reflect.Method;
 import java.util.Properties;
+import java.security.AccessController;
+import java.security.PrivilegedAction;
 
 import org.picocontainer.Characteristics;
 import org.picocontainer.ComponentAdapter;
@@ -185,8 +188,7 @@ public class AdaptingInjection extends AbstractInjectionFactory {
                                  Properties componentProperties,
                                  Object componentKey, ComponentAdapter<T> componentAdapter, Parameter... parameters) {
         if (injectionFieldAnnotated(componentImplementation)) {
-             componentAdapter =
-                new AnnotatedFieldInjection().createComponentAdapter(componentMonitor,
+             componentAdapter = new AnnotatedFieldInjection().createComponentAdapter(componentMonitor,
                                                                              lifecycleStrategy,
                                                                              componentProperties,
                                                                              componentKey,
@@ -196,12 +198,31 @@ public class AdaptingInjection extends AbstractInjectionFactory {
         return componentAdapter;
     }
 
-    private boolean injectionMethodAnnotated(Class<?> componentImplementation) {
-        return injectionAnnotated(componentImplementation.getDeclaredMethods());
+    private boolean injectionMethodAnnotated(final Class<?> componentImplementation) {
+        return (Boolean) AccessController.doPrivileged(new PrivilegedAction<Object>() {
+            public Object run() {
+                return injectionAnnotated(componentImplementation.getDeclaredMethods());
+            }
+        });
     }
 
-    private boolean injectionFieldAnnotated(Class<?> componentImplementation) {
-        return injectionAnnotated(componentImplementation.getDeclaredFields());
+    private boolean injectionFieldAnnotated(final Class<?> componentImplementation) {
+        return (Boolean) AccessController.doPrivileged(new PrivilegedAction<Object>() {
+            public Object run() {
+                if (componentImplementation.isInterface()) {
+                    return false;
+                }
+                Class impl = componentImplementation;
+                while (impl != Object.class) {
+                    boolean injAnnotated = injectionAnnotated(impl.getDeclaredFields());
+                    if (injAnnotated) {
+                        return true;
+                    }
+                    impl = impl.getSuperclass();
+                }
+                return false;
+            }
+        });
     }
     
     private boolean injectionAnnotated(AccessibleObject[] objects) {
