@@ -19,6 +19,7 @@ import java.lang.annotation.Annotation;
 import java.lang.reflect.AccessibleObject;
 import java.lang.reflect.Field;
 import java.lang.reflect.InvocationTargetException;
+import java.lang.reflect.Type;
 import java.security.AccessController;
 import java.security.PrivilegedAction;
 import java.util.ArrayList;
@@ -31,14 +32,14 @@ import java.util.List;
 @SuppressWarnings("serial")
 public class AnnotatedFieldInjector extends IterativeInjector {
 
- 	
-	private final Class<? extends Annotation> injectionAnnotation;
+
+    private final Class<? extends Annotation> injectionAnnotation;
 
     public AnnotatedFieldInjector(Object key,
                                   Class<?> impl,
                                   Parameter[] parameters,
                                   ComponentMonitor componentMonitor,
-                                  LifecycleStrategy lifecycleStrategy, 
+                                  LifecycleStrategy lifecycleStrategy,
                                   Class<? extends Annotation> injectionAnnotation, boolean useNames) {
 
         super(key, impl, parameters, componentMonitor, lifecycleStrategy, useNames);
@@ -48,16 +49,20 @@ public class AnnotatedFieldInjector extends IterativeInjector {
     protected void initializeInjectionMembersAndTypeLists() {
         injectionMembers = new ArrayList<AccessibleObject>();
         List<Annotation> bindingIds = new ArrayList<Annotation>();
-        final List<Class> typeList = new ArrayList<Class>();
-        final Field[] fields = getFields();
-        for (final Field field : fields) {
-            if (isAnnotatedForInjection(field)) {
-                injectionMembers.add(field);
-                typeList.add(box(field.getType()));
-                bindingIds.add(getBinding(field));
+        final List<Type> typeList = new ArrayList<Type>();
+        Class drillInto = getComponentImplementation();
+        while (drillInto != Object.class) {
+            final Field[] fields = getFields(drillInto);
+            for (final Field field : fields) {
+                if (isAnnotatedForInjection(field)) {
+                    injectionMembers.add(field);
+                    typeList.add(box(field.getType()));
+                    bindingIds.add(getBinding(field));
+                }
             }
+            drillInto = drillInto.getSuperclass();
         }
-        injectionTypes = typeList.toArray(new Class[0]);
+        injectionTypes = typeList.toArray(new Type[0]);
         bindings = bindingIds.toArray(new Annotation[0]);
     }
 
@@ -75,20 +80,21 @@ public class AnnotatedFieldInjector extends IterativeInjector {
         return field.getAnnotation(injectionAnnotation) != null;
     }
 
-    private Field[] getFields() {
+    private Field[] getFields(final Class clazz) {
         return (Field[]) AccessController.doPrivileged(new PrivilegedAction() {
             public Object run() {
-                return getComponentImplementation().getDeclaredFields();
+                return clazz.getDeclaredFields();
             }
         });
     }
 
 
-    protected void injectIntoMember(AccessibleObject member, Object componentInstance, Object toInject)
-        throws IllegalAccessException, InvocationTargetException {
+    protected Object injectIntoMember(AccessibleObject member, Object componentInstance, Object toInject)
+            throws IllegalAccessException, InvocationTargetException {
         Field field = (Field) member;
         field.setAccessible(true);
         field.set(componentInstance, toInject);
+        return null;
     }
 
     public String getDescriptor() {
@@ -102,4 +108,8 @@ public class AnnotatedFieldInjector extends IterativeInjector {
             }
         };
     }
+
+    protected Object memberInvocationReturn(Object lastReturn, AccessibleObject member, Object instance) {
+        return instance;
+    }
 }
