@@ -22,6 +22,7 @@ import java.lang.reflect.Constructor;
 import java.lang.reflect.InvocationTargetException;
 import java.lang.reflect.Modifier;
 import java.lang.reflect.Type;
+import java.lang.reflect.TypeVariable;
 import java.lang.annotation.Annotation;
 import java.security.AccessController;
 import java.security.PrivilegedAction;
@@ -102,26 +103,27 @@ public class ConstructorInjector<T> extends SingleMemberInjector<T> {
 
     protected Constructor<T> getGreediestSatisfiableConstructor(PicoContainer container) throws PicoCompositionException {
         final Set<Constructor> conflicts = new HashSet<Constructor>();
-        final Set<List<Class>> unsatisfiableDependencyTypes = new HashSet<List<Class>>();
+        final Set<List<Type>> unsatisfiableDependencyTypes = new HashSet<List<Type>>();
         if (sortedMatchingConstructors == null) {
             sortedMatchingConstructors = getSortedMatchingConstructors();
         }
         Constructor<T> greediestConstructor = null;
         int lastSatisfiableConstructorSize = -1;
-        Class<?> unsatisfiedDependencyType = null;
+        Type unsatisfiedDependencyType = null;
         for (final Constructor<T> sortedMatchingConstructor : sortedMatchingConstructors) {
             boolean failedDependency = false;
-            Class[] parameterTypes = sortedMatchingConstructor.getParameterTypes();
+            Type[] parameterTypes = sortedMatchingConstructor.getGenericParameterTypes();
+            fixParameterType(sortedMatchingConstructor, parameterTypes);
             Annotation[] bindings = getBindings(sortedMatchingConstructor.getParameterAnnotations());
             Parameter[] currentParameters = parameters != null ? parameters : createDefaultParameters(parameterTypes);
 
             // remember: all constructors with less arguments than the given parameters are filtered out already
             for (int j = 0; j < currentParameters.length; j++) {
-                // check whether this constructor is statisfiable
-                Class<?> boxed = box(parameterTypes[j]);
+                // check whether this constructor is satisfiable
+                Type boxed = box(parameterTypes[j]);
                 boolean un = useNames();
                 if (currentParameters[j].isResolvable(container, this, boxed,
-                    new ParameterNameBinding(getParanamer(), getComponentImplementation(),  sortedMatchingConstructor, j),
+                        new ParameterNameBinding(getParanamer(), getComponentImplementation(), sortedMatchingConstructor, j),
                         un, bindings[j])) {
                     continue;
                 }
@@ -163,6 +165,15 @@ public class ConstructorInjector<T> extends SingleMemberInjector<T> {
         return greediestConstructor;
     }
 
+    private void fixParameterType(Constructor<T> sortedMatchingConstructor, Type[] parameterTypes) {
+        for (int i = 0; i < parameterTypes.length; i++) {
+            Type parameterType = parameterTypes[i];
+            if (parameterType instanceof TypeVariable) {
+                parameterTypes[i] = sortedMatchingConstructor.getParameterTypes()[i];
+            }
+        }
+    }
+
     public T getComponentInstance(final PicoContainer container, Type into) throws PicoCompositionException {
         if (instantiationGuard == null) {
             instantiationGuard = new ThreadLocalCyclicDependencyGuard<T>() {
@@ -223,11 +234,10 @@ public class ConstructorInjector<T> extends SingleMemberInjector<T> {
         return inst;
     }
 
-    public void decorateComponentInstance(PicoContainer container, Type into, T instance) {
-    }
-
     protected Object[] getMemberArguments(PicoContainer container, final Constructor ctor) {
-        return super.getMemberArguments(container, ctor, ctor.getParameterTypes(), getBindings(ctor.getParameterAnnotations()));
+        Type[] parameterTypes = ctor.getGenericParameterTypes();
+        fixParameterType(ctor, parameterTypes);
+        return super.getMemberArguments(container, ctor, parameterTypes, getBindings(ctor.getParameterAnnotations()));
     }
 
     private List<Constructor<T>> getSortedMatchingConstructors() {
