@@ -26,12 +26,13 @@ import org.picocontainer.PicoVisitor;
 
 /**
  * <p>
- * A Injector which provides an custom instance in a factory style
+ * An Injector which provides an custom instance in a factory style
  * </p>
  *
  * @author Paul Hammant
  */
 public abstract class FactoryInjector<T> implements Injector<T> {
+
     private Class key;
 
     public FactoryInjector() throws PicoCompositionException {
@@ -141,12 +142,12 @@ public abstract class FactoryInjector<T> implements Injector<T> {
         throw new UnsupportedOperationException();
     }
 
-    public abstract T getComponentInstance(PicoContainer container, Type clazz);
+    public abstract T getComponentInstance(PicoContainer container, Type into);
 
-    public void decorateComponentInstance(PicoContainer container, Type into, T instance) {
+    public Object decorateComponentInstance(PicoContainer container, Type into, T instance) {
+        return null;
     }
 
-
     public void verify(PicoContainer container) {
     }
 
@@ -167,6 +168,9 @@ public abstract class FactoryInjector<T> implements Injector<T> {
         return false;
     }
 
-    public static class CantWorkItOut {}
+    public static class CantWorkItOut {
+        private CantWorkItOut() {
+        }
+    }
 
 }
\ No newline at end of file
