@@ -23,7 +23,9 @@ import java.util.HashSet;
 import java.util.List;
 import java.util.Set;
 
+import com.thoughtworks.paranamer.Paranamer;
 import com.thoughtworks.paranamer.CachingParanamer;
+import com.thoughtworks.paranamer.AdaptiveParanamer;
 
 /**
  * Injection will happen iteratively after component instantiation
@@ -31,12 +33,10 @@ import com.thoughtworks.paranamer.CachingParanamer;
 public abstract class IterativeInjector<T> extends AbstractInjector<T> {
     private transient ThreadLocalCyclicDependencyGuard instantiationGuard;
     protected transient List<AccessibleObject> injectionMembers;
-    protected transient Class[] injectionTypes;
+    protected transient Type[] injectionTypes;
     protected transient Annotation[] bindings;
 
-
-    private transient CachingParanamer paranamer = new CachingParanamer();
-
+    private transient Paranamer paranamer;
 
     /**
      * Constructs a IterativeInjector
@@ -56,10 +56,6 @@ public abstract class IterativeInjector<T> extends AbstractInjector<T> {
         super(componentKey, componentImplementation, parameters, monitor, lifecycleStrategy, useNames);
     }
 
-    protected CachingParanamer getParanamer() {
-        return paranamer;
-    }
-
     protected Constructor getConstructor()  {
         Object retVal = AccessController.doPrivileged(new PrivilegedAction() {
             public Object run() {
@@ -89,7 +85,7 @@ public abstract class IterativeInjector<T> extends AbstractInjector<T> {
         final Parameter[] currentParameters = parameters != null ? parameters : createDefaultParameters(injectionTypes);
         final Set<Integer> nonMatchingParameterPositions = matchParameters(container, matchingParameterList, currentParameters);
 
-        final Set<Class> unsatisfiableDependencyTypes = new HashSet<Class>();
+        final Set<Type> unsatisfiableDependencyTypes = new HashSet<Type>();
         for (int i = 0; i < matchingParameterList.size(); i++) {
             if (matchingParameterList.get(i) == null) {
                 unsatisfiableDependencyTypes.add(injectionTypes[i]);
@@ -127,10 +123,13 @@ public abstract class IterativeInjector<T> extends AbstractInjector<T> {
     }
 
     protected NameBinding makeParameterNameImpl(AccessibleObject member) {
+        if (paranamer == null) {
+            paranamer = new CachingParanamer(new AdaptiveParanamer());
+        }
         return new ParameterNameBinding(paranamer, getComponentImplementation(),  member, 0);
     }
 
-    protected void unsatisfiedDependencies(PicoContainer container, Set<Class> unsatisfiableDependencyTypes) {
+    protected void unsatisfiedDependencies(PicoContainer container, Set<Type> unsatisfiableDependencyTypes) {
         throw new UnsatisfiableDependenciesException(this, null, unsatisfiableDependencyTypes, container);
     }
 
@@ -142,7 +141,6 @@ public abstract class IterativeInjector<T> extends AbstractInjector<T> {
                     final Parameter[] matchingParameters = getMatchingParameterListForSetters(guardedContainer);
                     Object componentInstance = makeInstance(container, constructor, currentMonitor());
                     return decorateComponentInstance(matchingParameters, currentMonitor(), componentInstance, container, guardedContainer);
-
                 }
             };
         }
@@ -153,20 +151,20 @@ public abstract class IterativeInjector<T> extends AbstractInjector<T> {
     private Object decorateComponentInstance(Parameter[] matchingParameters, ComponentMonitor componentMonitor, Object componentInstance, PicoContainer container, PicoContainer guardedContainer) {
         AccessibleObject member = null;
         Object injected[] = new Object[injectionMembers.size()];
+        Object lastReturn = null;
         try {
             for (int i = 0; i < injectionMembers.size(); i++) {
                 member = injectionMembers.get(i);
                 componentMonitor.invoking(container, this, (Member) member, componentInstance);
-                if (matchingParameters[i] == null) {
-                    continue;
+                if (matchingParameters[i] != null) {
+                    Object toInject = matchingParameters[i].resolveInstance(guardedContainer, this, injectionTypes[i],
+                                                                            makeParameterNameImpl(injectionMembers.get(i)),
+                                                                            useNames(), bindings[i]);
+                    lastReturn = injectIntoMember(member, componentInstance, toInject);
+                    injected[i] = toInject;
                 }
-                Object toInject = matchingParameters[i].resolveInstance(guardedContainer, this, injectionTypes[i],
-                                                                        makeParameterNameImpl(injectionMembers.get(i)),
-                                                                        useNames(), bindings[i]);
-                injectIntoMember(member, componentInstance, toInject);
-                injected[i] = toInject;
             }
-            return componentInstance;
+            return memberInvocationReturn(lastReturn, member, componentInstance);
         } catch (InvocationTargetException e) {
             return caughtInvocationTargetException(componentMonitor, (Member) member, componentInstance, e);
         } catch (IllegalAccessException e) {
@@ -174,6 +172,8 @@ public abstract class IterativeInjector<T> extends AbstractInjector<T> {
         }
     }
 
+    protected abstract Object memberInvocationReturn(Object lastReturn, AccessibleObject member, Object instance);
+
     private Object makeInstance(PicoContainer container, Constructor constructor, ComponentMonitor componentMonitor) {
         long startTime = System.currentTimeMillis();
         Constructor constructorToUse = componentMonitor.instantiating(container,
@@ -203,7 +203,8 @@ public abstract class IterativeInjector<T> extends AbstractInjector<T> {
         return componentInstance;
     }
 
-    public void decorateComponentInstance(final PicoContainer container, Type into, final T instance) {
+    @Override
+    public Object decorateComponentInstance(final PicoContainer container, Type into, final T instance) {
         if (instantiationGuard == null) {
             instantiationGuard = new ThreadLocalCyclicDependencyGuard() {
                 public Object run() {
@@ -213,11 +214,10 @@ public abstract class IterativeInjector<T> extends AbstractInjector<T> {
             };
         }
         instantiationGuard.setGuardedContainer(container);
-        instantiationGuard.observe(getComponentImplementation());
-
+        return instantiationGuard.observe(getComponentImplementation());
     }
 
-    protected abstract void injectIntoMember(AccessibleObject member, Object componentInstance, Object toInject) throws IllegalAccessException, InvocationTargetException;
+    protected abstract Object injectIntoMember(AccessibleObject member, Object componentInstance, Object toInject) throws IllegalAccessException, InvocationTargetException;
 
     @Override
     public void verify(final PicoContainer container) throws PicoCompositionException {
@@ -240,7 +240,7 @@ public abstract class IterativeInjector<T> extends AbstractInjector<T> {
     protected void initializeInjectionMembersAndTypeLists() {
         injectionMembers = new ArrayList<AccessibleObject>();
         List<Annotation> bingingIds = new ArrayList<Annotation>();
-        final List<Class> typeList = new ArrayList<Class>();
+        final List<Type> typeList = new ArrayList<Type>();
         final Method[] methods = getMethods();
         for (final Method method : methods) {
             final Class[] parameterTypes = method.getParameterTypes();
@@ -254,7 +254,7 @@ public abstract class IterativeInjector<T> extends AbstractInjector<T> {
                 }
             }
         }
-        injectionTypes = typeList.toArray(new Class[0]);
+        injectionTypes = typeList.toArray(new Type[0]);
         bindings = bingingIds.toArray(new Annotation[0]);
     }
 
@@ -270,9 +270,9 @@ public abstract class IterativeInjector<T> extends AbstractInjector<T> {
             return null;
 
         }
+        //TODO - what's this ?
         if (parameterAnnotations != null) {
             //return ((Bind) method.getAnnotation(Bind.class)).id();
-            System.out.println("");
         }
         return null;
 
