@@ -12,12 +12,16 @@ package org.picocontainer.injectors;
 import java.lang.reflect.InvocationTargetException;
 import java.lang.reflect.Method;
 import java.lang.reflect.Type;
+import java.lang.reflect.Member;
+import java.lang.reflect.AccessibleObject;
+import java.lang.annotation.Annotation;
 
 import org.picocontainer.ComponentMonitor;
 import org.picocontainer.LifecycleStrategy;
 import org.picocontainer.Parameter;
 import org.picocontainer.PicoCompositionException;
 import org.picocontainer.PicoContainer;
+import org.picocontainer.annotations.Nullable;
 
 /**
  * Injection will happen through a single method for the component.
@@ -108,37 +112,40 @@ public class MethodInjector<T> extends SingleMemberInjector<T> {
         return super.getMemberArguments(container, method, method.getParameterTypes(), getBindings(method.getParameterAnnotations()));
     }
 
-    public void decorateComponentInstance(final PicoContainer container, Type into, final T instance) {
+    @Override
+    public Object decorateComponentInstance(final PicoContainer container, final Type into, final T instance) {
         if (instantiationGuard == null) {
             instantiationGuard = new ThreadLocalCyclicDependencyGuard() {
                 public Object run() {
                     Method method = getInjectorMethod();
-                    Object inst = null;
-                    Object[] parameters = null;
-                    parameters = getMemberArguments(guardedContainer, method);
-                    return invokeMethod(method, parameters, instance, container);
+                    if (method.getDeclaringClass().isAssignableFrom(instance.getClass())) {
+                        Object[] parameters = getMemberArguments(guardedContainer, method);
+                        return invokeMethod(method, parameters, instance, container);
+                    }
+                    return null;
                 }
             };
         }
         instantiationGuard.setGuardedContainer(container);
-        instantiationGuard.observe(getComponentImplementation());
+        return instantiationGuard.observe(getComponentImplementation());
 
     }
 
     private Object invokeMethod(Method method, Object[] parameters, T instance, PicoContainer container) {
         try {
+            currentMonitor().invoking(container, MethodInjector.this, (Member) method, instance);
             return method.invoke(instance, parameters);
         } catch (IllegalAccessException e) {
             return caughtIllegalAccessException(currentMonitor(), method, instance, e);
         } catch (InvocationTargetException e) {
-            currentMonitor().instantiationFailed(container, MethodInjector.this, null, e);
+            currentMonitor().invocationFailed(method, instance, e);
             if (e.getTargetException() instanceof RuntimeException) {
                 throw (RuntimeException) e.getTargetException();
             } else if (e.getTargetException() instanceof Error) {
                 throw (Error) e.getTargetException();
             }
+            return null;
         }
-        return null;
     }
 
 
@@ -167,4 +174,33 @@ public class MethodInjector<T> extends SingleMemberInjector<T> {
         return "MethodInjector-";
     }
 
+    protected boolean isNullParamAllowed(AccessibleObject member, int i) {
+        Annotation[] annotations = ((Method) member).getParameterAnnotations()[i];
+        for (Annotation annotation : annotations) {
+            if (annotation instanceof Nullable) {
+                return true;
+            }
+        }
+        return false;
+    }
+
+
+    public static class ByReflectionMethod extends MethodInjector {
+        private final Method injectionMethod;
+
+        public ByReflectionMethod(Object componentKey, Class componentImplementation, Parameter[] parameters, ComponentMonitor monitor, LifecycleStrategy lifecycleStrategy, Method injectionMethod, boolean useNames) throws NotConcreteRegistrationException {
+            super(componentKey, componentImplementation, parameters, monitor, lifecycleStrategy, null, useNames);
+            this.injectionMethod = injectionMethod;
+        }
+        
+        @Override
+        protected Method getInjectorMethod() {
+            return injectionMethod;
+        }
+        public String getDescriptor() {
+            return "ReflectionMethodInjector[" + injectionMethod + "]-";
+        }
+
+    }
+
 }
\ No newline at end of file
