@@ -12,11 +12,16 @@ import org.picocontainer.ComponentMonitor;
 import org.picocontainer.LifecycleStrategy;
 import org.picocontainer.Parameter;
 import org.picocontainer.PicoContainer;
+import org.picocontainer.PicoCompositionException;
 import org.picocontainer.annotations.Bind;
 
 import java.lang.annotation.Annotation;
 import java.lang.reflect.AccessibleObject;
+import java.lang.reflect.Type;
+
 import com.thoughtworks.paranamer.CachingParanamer;
+import com.thoughtworks.paranamer.AdaptiveParanamer;
+import com.thoughtworks.paranamer.Paranamer;
 
 /**
  * Injection will happen in a single member function on the component.
@@ -26,7 +31,7 @@ import com.thoughtworks.paranamer.CachingParanamer;
  */
 public abstract class SingleMemberInjector<T> extends AbstractInjector<T> {
 
-    private transient CachingParanamer paranamer = new CachingParanamer();
+    private transient Paranamer paranamer;
 
     public SingleMemberInjector(Object componentKey,
                                 Class componentImplementation,
@@ -36,61 +41,50 @@ public abstract class SingleMemberInjector<T> extends AbstractInjector<T> {
         super(componentKey, componentImplementation, parameters, monitor, lifecycleStrategy, useNames);
     }
 
-    protected CachingParanamer getParanamer() {
+    protected Paranamer getParanamer() {
+        if (paranamer == null) {
+            paranamer = new CachingParanamer(new AdaptiveParanamer());
+        }
         return paranamer;
     }
 
-
-    /**
-     * TODO: shall it box everything?  a bit too few for me (konstantin)
-     */
-    protected Class box(Class parameterType) {
-        if (parameterType.isPrimitive()) {
-            String parameterTypeName = parameterType.getName();
-            if (parameterTypeName == "int") {
-                return Integer.class;
-            } else if (parameterTypeName == "boolean") {
-                return Boolean.class;
-            } else if (parameterTypeName == "long") {
-                return Long.class;
-            } else if (parameterTypeName == "float") {
-                return Float.class;
-            } else if (parameterTypeName == "double") {
-                return Double.class;
-            } else if (parameterTypeName == "char") {
-                return Character.class;
-            } else if (parameterTypeName == "byte") {
-                return Byte.class;
-            } else if (parameterTypeName == "short") {
-                return Short.class;
-            }
-        }
-        return parameterType;
-    }
-
-
     @SuppressWarnings("unchecked")
-    protected Object[] getMemberArguments(PicoContainer container, final AccessibleObject member, final Class[] parameterTypes, final Annotation[] bindings) {
-        for (int i = 0; i < parameterTypes.length; i++) {
-            parameterTypes[i] = box(parameterTypes[i]);
-
-        }
+    protected Object[] getMemberArguments(PicoContainer container, final AccessibleObject member, final Type[] parameterTypes, final Annotation[] bindings) {
+        boxParameters(parameterTypes);
         Object[] result = new Object[parameterTypes.length];
         Parameter[] currentParameters = parameters != null ? parameters : createDefaultParameters(parameterTypes);
 
         for (int i = 0; i < currentParameters.length; i++) {
-            result[i] = currentParameters[i].resolveInstance(container, this, parameterTypes[i],
-                new ParameterNameBinding(paranamer, getComponentImplementation(), member, i), useNames(), bindings[i]);
+            result[i] = getParameter(container, member, i, parameterTypes[i], bindings[i], currentParameters[i]);
+        }
+
+        return result;
+    }
+
+    private void boxParameters(Type[] parameterTypes) {
+        for (int i = 0; i < parameterTypes.length; i++) {
+            parameterTypes[i] = box(parameterTypes[i]);
+        }
+    }
+
+    private Object getParameter(PicoContainer container, AccessibleObject member, int i, Type parameterType, Annotation binding, Parameter currentParameter) {
+        ParameterNameBinding expectedNameBinding = new ParameterNameBinding(getParanamer(), getComponentImplementation(), member, i);
+        Object result = currentParameter.resolveInstance(container, this, parameterType, expectedNameBinding, useNames(), binding);
+        if (result == null && !isNullParamAllowed(member, i)) {
+            throw new ParameterCannotBeNullException(i, member, expectedNameBinding.getName());
         }
         return result;
     }
 
+    protected boolean isNullParamAllowed(AccessibleObject member, int i) {
+        return false;
+    }
+
     protected Annotation[] getBindings(Annotation[][] annotationss) {
         Annotation[] retVal = new Annotation[annotationss.length];
         for (int i = 0; i < annotationss.length; i++) {
             Annotation[] annotations = annotationss[i];
-            for (int j = 0; j < annotations.length; j++) {
-                Annotation annotation = annotations[j];
+            for (Annotation annotation : annotations) {
                 if (annotation.annotationType().getAnnotation(Bind.class) != null) {
                     retVal[i] = annotation;
                     break;
@@ -100,5 +94,15 @@ public abstract class SingleMemberInjector<T> extends AbstractInjector<T> {
         return retVal;
     }
 
+    public static class ParameterCannotBeNullException extends PicoCompositionException {
+        private final String name;
+        private ParameterCannotBeNullException(int ix, AccessibleObject member, String name) {
+            super("Parameter " + ix + " of '" + member + "' named '" + name + "' cannot be null");
+            this.name = name;
+        }
+        public String getParameterName() {
+            return name;
+        }
+    }
 
 }
