@@ -13,6 +13,7 @@ import java.lang.annotation.Annotation;
 import java.lang.reflect.AccessibleObject;
 import java.lang.reflect.Field;
 import java.lang.reflect.InvocationTargetException;
+import java.lang.reflect.Type;
 import java.security.AccessController;
 import java.security.PrivilegedAction;
 import java.util.ArrayList;
@@ -53,7 +54,7 @@ public class TypedFieldInjector extends IterativeInjector {
     protected void initializeInjectionMembersAndTypeLists() {
         injectionMembers = new ArrayList<AccessibleObject>();
         List<Annotation> bindingIds = new ArrayList<Annotation>();
-        final List<Class> typeList = new ArrayList<Class>();
+        final List<Type> typeList = new ArrayList<Type>();
         final Field[] fields = getFields();
         for (final Field field : fields) {
             if (isTypedForInjection(field)) {
@@ -62,7 +63,7 @@ public class TypedFieldInjector extends IterativeInjector {
                 bindingIds.add(getBinding(field));
             }
         }
-        injectionTypes = typeList.toArray(new Class[0]);
+        injectionTypes = typeList.toArray(new Type[0]);
         bindings = bindingIds.toArray(new Annotation[0]);
     }
 
@@ -89,11 +90,12 @@ public class TypedFieldInjector extends IterativeInjector {
     }
 
 
-    protected void injectIntoMember(AccessibleObject member, Object componentInstance, Object toInject)
+    protected Object injectIntoMember(AccessibleObject member, Object componentInstance, Object toInject)
         throws IllegalAccessException, InvocationTargetException {
         Field field = (Field) member;
         field.setAccessible(true);
         field.set(componentInstance, toInject);
+        return null;
     }
 
     public String getDescriptor() {
@@ -108,6 +110,10 @@ public class TypedFieldInjector extends IterativeInjector {
         };
     }
 
+    protected Object memberInvocationReturn(Object lastReturn, AccessibleObject member, Object instance) {
+        return instance;
+    }
+
     List<String> getInjectionFieldTypes() {
         return Collections.unmodifiableList(classes);
     }
