@@ -20,7 +20,8 @@ import org.picocontainer.ComponentMonitor;
 import org.picocontainer.ComponentMonitorStrategy;
 import org.picocontainer.MutablePicoContainer;
 import org.picocontainer.PicoContainer;
-import org.picocontainer.injectors.AbstractInjector;
+import org.picocontainer.Injector;
+import org.picocontainer.Behavior;
 
 /**
  * <p>
@@ -112,8 +113,12 @@ public class AbstractComponentMonitor implements ComponentMonitor, ComponentMoni
         return delegate.noComponentFound(container, componentKey);
     }
 
-    public AbstractInjector newInjectionFactory(AbstractInjector abstractInjector) {
-        return abstractInjector;
+    public Injector newInjector(Injector injector) {
+        return injector;
+    }
+
+    public Behavior newBehavior(Behavior behavior) {
+        return behavior;
     }
 
     /**
