@@ -27,7 +27,8 @@ import org.picocontainer.ComponentAdapter;
 import org.picocontainer.ComponentMonitor;
 import org.picocontainer.MutablePicoContainer;
 import org.picocontainer.PicoContainer;
-import org.picocontainer.injectors.AbstractInjector;
+import org.picocontainer.Injector;
+import org.picocontainer.Behavior;
 
 /**
  * A {@link ComponentMonitor} which writes to a {@link OutputStream}. 
@@ -39,8 +40,7 @@ import org.picocontainer.injectors.AbstractInjector;
  * @todo  After serialization, the output printstream is null.  
  */
 @SuppressWarnings("serial")
-public final class ConsoleComponentMonitor implements ComponentMonitor, Serializable {
-
+public class ConsoleComponentMonitor implements ComponentMonitor, Serializable {
 
 	/**
 	 * The outgoing print stream.
@@ -139,8 +139,13 @@ public final class ConsoleComponentMonitor implements ComponentMonitor, Serializ
         return delegate.noComponentFound(container, componentKey);
     }
 
-    public AbstractInjector newInjectionFactory(AbstractInjector abstractInjector) {
-        return delegate.newInjectionFactory(abstractInjector);
+    public Injector newInjector(Injector injector) {
+        return delegate.newInjector(injector);
+    }
+
+    /** {@inheritDoc} **/
+    public Behavior newBehavior(Behavior behavior) {
+        return delegate.newBehavior(behavior);
     }
 
 }
