@@ -22,7 +22,8 @@ import org.picocontainer.MutablePicoContainer;
 import org.picocontainer.PicoContainer;
 import org.picocontainer.PicoException;
 import org.picocontainer.PicoLifecycleException;
-import org.picocontainer.injectors.AbstractInjector;
+import org.picocontainer.Injector;
+import org.picocontainer.Behavior;
 
 /**
  * A {@link ComponentMonitor} which collects lifecycle failures
@@ -104,8 +105,13 @@ public final class LifecycleComponentMonitor implements ComponentMonitor {
         return delegate.noComponentFound(container, componentKey);
     }
 
-    public AbstractInjector newInjectionFactory(AbstractInjector abstractInjector) {
-        return delegate.newInjectionFactory(abstractInjector);
+    public Injector newInjector(Injector injector) {
+        return delegate.newInjector(injector);
+    }
+
+    /** {@inheritDoc} **/
+    public Behavior newBehavior(Behavior behavior) {
+        return delegate.newBehavior(behavior);
     }
 
 
