@@ -20,7 +20,8 @@ import org.picocontainer.ComponentMonitor;
 import org.picocontainer.MutablePicoContainer;
 import org.picocontainer.PicoContainer;
 import org.picocontainer.PicoLifecycleException;
-import org.picocontainer.injectors.AbstractInjector;
+import org.picocontainer.Injector;
+import org.picocontainer.Behavior;
 
 /**
  * A {@link ComponentMonitor} which does nothing. 
@@ -79,8 +80,14 @@ public class NullComponentMonitor implements ComponentMonitor, Serializable {
         return null;
     }
 
-    public AbstractInjector newInjectionFactory(AbstractInjector abstractInjector) {
-        return abstractInjector;  
+    public Injector newInjector(Injector injector) {
+        return injector;
     }
 
+    /** {@inheritDoc} **/
+    public Behavior newBehavior(Behavior behavior) {
+        return behavior;
+    }
+    
+
 }
