@@ -13,7 +13,8 @@ import java.io.File;
 import java.io.Serializable;
 import java.lang.reflect.Constructor;
 import java.lang.reflect.InvocationTargetException;
-import java.lang.reflect.Method;
+import java.lang.reflect.ParameterizedType;
+import java.lang.reflect.Type;
 import java.lang.annotation.Annotation;
 import java.util.HashMap;
 import java.util.HashSet;
@@ -26,6 +27,7 @@ import org.picocontainer.Parameter;
 import org.picocontainer.NameBinding;
 import org.picocontainer.PicoContainer;
 import org.picocontainer.PicoVisitor;
+import org.picocontainer.DefaultPicoContainer;
 import org.picocontainer.injectors.AbstractInjector;
 
 /**
@@ -47,25 +49,7 @@ public class BasicComponentParameter implements Parameter, Serializable {
     private static interface Converter {
         Object convert(String paramValue);
     }
-    private static class ValueOfConverter implements Converter {
-        private Method m;
-        private ValueOfConverter(Class clazz) {
-            try {
-                m = clazz.getMethod("valueOf", String.class);
-            } catch (NoSuchMethodException e) {
-            }
-        }
 
-        public Object convert(String paramValue) {
-            try {
-                return m.invoke(null, paramValue);
-            } catch (IllegalAccessException e) {
-            } catch (InvocationTargetException e) {
-            }
-            return null;
-
-        }
-    }
     private static class NewInstanceConverter implements Converter {
         private Constructor c;
 
@@ -95,15 +79,51 @@ public class BasicComponentParameter implements Parameter, Serializable {
 
     private static final Map<Class, Converter> stringConverters = new HashMap<Class, Converter>();
     static {
-        stringConverters.put(Integer.class, new ValueOfConverter(Integer.class));
-        stringConverters.put(Double.class, new ValueOfConverter(Double.class));
-        stringConverters.put(Boolean.class, new ValueOfConverter(Boolean.class));
-        stringConverters.put(Long.class, new ValueOfConverter(Long.class));
-        stringConverters.put(Float.class, new ValueOfConverter(Float.class));
-        stringConverters.put(Character.class, new ValueOfConverter(Character.class));
-        stringConverters.put(Byte.class, new ValueOfConverter(Byte.class));
-        stringConverters.put(Byte.class, new ValueOfConverter(Short.class));
-        stringConverters.put(File.class, new NewInstanceConverter(File.class));
+        stringConverters.put(Integer.class, new Converter(){
+            public Object convert(String paramValue) {
+                return Integer.valueOf(paramValue);
+            }
+        });
+        stringConverters.put(Double.class, new Converter() {
+            public Object convert(String paramValue) {
+                return Double.valueOf(paramValue);
+            }
+        });
+        stringConverters.put(Boolean.class, new Converter(){
+            public Object convert(String paramValue) {
+                return Boolean.valueOf(paramValue);
+            }
+        });
+        stringConverters.put(Long.class, new Converter() {
+            public Object convert(String paramValue) {
+                return Long.valueOf(paramValue);
+            }
+        });
+        stringConverters.put(Float.class, new Converter() {
+            public Object convert(String paramValue) {
+                return Float.valueOf(paramValue);
+            }
+        });
+        stringConverters.put(Character.class, new Converter() {
+            public Object convert(String paramValue) {
+                return paramValue.charAt(0);
+            }
+        });
+        stringConverters.put(Byte.class, new Converter() {
+            public Object convert(String paramValue) {
+                return Byte.valueOf(paramValue);
+            }
+        });
+        stringConverters.put(Short.class, new Converter() {
+            public Object convert(String paramValue) {
+                return Short.valueOf(paramValue);
+            }
+        });
+        stringConverters.put(File.class, new Converter() {
+            public Object convert(String paramValue) {
+                return new File(paramValue);
+            }
+        });
 
     }
 
@@ -131,37 +151,59 @@ public class BasicComponentParameter implements Parameter, Serializable {
      * @see Parameter#isResolvable(PicoContainer, ComponentAdapter, Class, NameBinding ,boolean, Annotation)
      */
     public boolean isResolvable(PicoContainer container,
-                                ComponentAdapter adapter,
-                                Class expectedType,
+                                ComponentAdapter<?> adapter,
+                                Type expectedType,
                                 NameBinding expectedNameBinding, boolean useNames, Annotation binding) {
-        return resolveAdapter(container, adapter, (Class<?>)expectedType, expectedNameBinding, useNames, binding) != null;
+    	
+    	Class<?> resolvedClassType = null;
+        // TODO take this out for Pico3
+        if (!(expectedType instanceof Class)) {
+        	if (expectedType instanceof ParameterizedType) {
+        		resolvedClassType = (Class<?>) ((ParameterizedType)expectedType).getRawType();
+        	} else {
+        		return false;
+        	}
+        } else {
+        	resolvedClassType = (Class<?>)expectedType;
+        }
+        assert resolvedClassType != null;
+        
+        return resolveAdapter(container, adapter, resolvedClassType, expectedNameBinding, useNames, binding) != null;
     }
 
-    public <T> T resolveInstance(PicoContainer container,
-                                 ComponentAdapter adapter,
-                                 Class<T> expectedType,
+    public Object resolveInstance(PicoContainer container,
+                                 ComponentAdapter<?> adapter,
+                                 Type expectedType,
                                  NameBinding expectedNameBinding, boolean useNames, Annotation binding) {
-        final ComponentAdapter componentAdapter =
+        ComponentAdapter componentAdapter =
             resolveAdapter(container, adapter, (Class<?>)expectedType, expectedNameBinding, useNames, binding);
+        if (componentAdapter == null && useNames) {
+            componentAdapter = container.getComponentAdapter(expectedNameBinding.getName());             
+        }
         if (componentAdapter != null) {
-            Object o = container.getComponent(componentAdapter.getComponentKey(), adapter.getComponentImplementation());
+            Object o;
+            if (componentAdapter instanceof DefaultPicoContainer.LateInstance) {
+                o = ((DefaultPicoContainer.LateInstance) componentAdapter).getComponentInstance();
+            } else {
+                o = container.getComponent(componentAdapter.getComponentKey(), adapter.getComponentImplementation());
+            }
             if (o instanceof String && expectedType != String.class) {
                 Converter converter = stringConverters.get(expectedType);
-                return (T) converter.convert((String) o);
+                return converter.convert((String) o);
             }
-            return (T) o;
+            return o;
         }
         return null;
     }
 
     public void verify(PicoContainer container,
-                       ComponentAdapter adapter,
-                       Class expectedType,
+                       ComponentAdapter<?> adapter,
+                       Type expectedType,
                        NameBinding expectedNameBinding, boolean useNames, Annotation binding) {
         final ComponentAdapter componentAdapter =
             resolveAdapter(container, adapter, (Class<?>)expectedType, expectedNameBinding, useNames, binding);
         if (componentAdapter == null) {
-            final Set<Class> set = new HashSet<Class>();
+            final Set<Type> set = new HashSet<Type>();
             set.add(expectedType);
             throw new AbstractInjector.UnsatisfiableDependenciesException(adapter, null, set, container);
         }
@@ -208,7 +250,6 @@ public class BasicComponentParameter implements Parameter, Serializable {
         if (result == null) {
             return null;
         }
-
         if (!type.isAssignableFrom(result.getComponentImplementation())) {
             if (!(result.getComponentImplementation() == String.class && stringConverters.containsKey(type))) {
                 return null;
@@ -222,8 +263,7 @@ public class BasicComponentParameter implements Parameter, Serializable {
         return (ComponentAdapter<T>)componentAdapter;
     }
 
-    private <T> ComponentAdapter<T> getTargetAdapter(PicoContainer container,
-                                                     Class<T> expectedType,
+    private <T> ComponentAdapter<T> getTargetAdapter(PicoContainer container, Class<T> expectedType,
                                                      NameBinding expectedNameBinding,
                                                      ComponentAdapter excludeAdapter, boolean useNames, Annotation binding) {
         if (componentKey != null) {
@@ -232,49 +272,67 @@ public class BasicComponentParameter implements Parameter, Serializable {
         } else if (excludeAdapter == null) {
             return container.getComponentAdapter(expectedType, (NameBinding) null);
         } else {
+            return findTargetAdapter(container, expectedType, expectedNameBinding, excludeAdapter, useNames, binding);
+        }
+    }
 
-            Object excludeKey = excludeAdapter.getComponentKey();
-            ComponentAdapter byKey = container.getComponentAdapter((Object)expectedType);
-            if (byKey != null && !excludeKey.equals(byKey.getComponentKey())) {
-                return typeComponentAdapter(byKey);
-            }
-            if (useNames) {
-                ComponentAdapter found = container.getComponentAdapter(expectedNameBinding.getName());
-                if ((found != null)
-                    && areCompatible(expectedType, found)
-                    && found != excludeAdapter) {
-                    return (ComponentAdapter<T>) found;                    
-                }
-            }
-            List<ComponentAdapter<T>> found = binding == null ? container.getComponentAdapters(expectedType) :
-                                              container.getComponentAdapters(expectedType, binding.annotationType());
-            ComponentAdapter exclude = null;
-            for (ComponentAdapter work : found) {
-                if (work.getComponentKey().equals(excludeKey)) {
-                    exclude = work;
-                }
-            }
-            found.remove(exclude);
-            if (found.size() == 0) {
-                if (container.getParent() != null) {
-                    if (binding != null) {
-                        return container.getParent().getComponentAdapter(expectedType, binding.getClass());
-                    } else {
-                        return container.getParent().getComponentAdapter(expectedType, expectedNameBinding);
-                    }
-                } else {
-                    return null;
-                }
-            } else if (found.size() == 1) {
-                return found.get(0);
-            } else {
-                Class[] foundClasses = new Class[found.size()];
-                for (int i = 0; i < foundClasses.length; i++) {
-                    foundClasses[i] = found.get(i).getComponentImplementation();
-                }
-                throw new AbstractInjector.AmbiguousComponentResolutionException(expectedType, foundClasses);
+    private <T> ComponentAdapter<T> findTargetAdapter(PicoContainer container, Class<T> expectedType,
+                                                      NameBinding expectedNameBinding, ComponentAdapter excludeAdapter,
+                                                      boolean useNames, Annotation binding) {
+        Object excludeKey = excludeAdapter.getComponentKey();
+        ComponentAdapter byKey = container.getComponentAdapter((Object)expectedType);
+        if (byKey != null && !excludeKey.equals(byKey.getComponentKey())) {
+            return typeComponentAdapter(byKey);
+        }
+        if (useNames) {
+            ComponentAdapter found = container.getComponentAdapter(expectedNameBinding.getName());
+            if ((found != null) && areCompatible(expectedType, found) && found != excludeAdapter) {
+                return (ComponentAdapter<T>) found;
             }
         }
+        List<ComponentAdapter<T>> found = binding == null ? container.getComponentAdapters(expectedType) :
+                                          container.getComponentAdapters(expectedType, binding.annotationType());
+        removeExcludedAdapterIfApplicable(excludeKey, found);
+        if (found.size() == 0) {
+            return noMatchingAdaptersFound(container, expectedType, expectedNameBinding, binding);
+        } else if (found.size() == 1) {
+            return found.get(0);
+        } else {
+            throw tooManyMatchingAdaptersFound(expectedType, found);
+        }
+    }
+
+    private <T> ComponentAdapter<T> noMatchingAdaptersFound(PicoContainer container, Class<T> expectedType,
+                                                            NameBinding expectedNameBinding, Annotation binding) {
+        if (container.getParent() != null) {
+            if (binding != null) {
+                return container.getParent().getComponentAdapter(expectedType, binding.getClass());
+            } else {
+                return container.getParent().getComponentAdapter(expectedType, expectedNameBinding);
+            }
+        } else {
+            return null;
+        }
+    }
+
+    private <T> AbstractInjector.AmbiguousComponentResolutionException tooManyMatchingAdaptersFound(Class<T> expectedType, List<ComponentAdapter<T>> found) {
+        Class[] foundClasses = new Class[found.size()];
+        for (int i = 0; i < foundClasses.length; i++) {
+            foundClasses[i] = found.get(i).getComponentImplementation();
+        }
+        AbstractInjector.AmbiguousComponentResolutionException exception = new AbstractInjector.AmbiguousComponentResolutionException(expectedType, foundClasses);
+        return exception;
+    }
+
+    private <T> void removeExcludedAdapterIfApplicable(Object excludeKey, List<ComponentAdapter<T>> found) {
+        ComponentAdapter exclude = null;
+        for (ComponentAdapter work : found) {
+            if (work.getComponentKey().equals(excludeKey)) {
+                exclude = work;
+                break;
+            }
+        }
+        found.remove(exclude);
     }
 
     private <T> boolean areCompatible(Class<T> expectedType, ComponentAdapter found) {
