@@ -20,6 +20,8 @@ import org.picocontainer.NameBinding;
 
 import java.io.Serializable;
 import java.lang.reflect.Field;
+import java.lang.reflect.ParameterizedType;
+import java.lang.reflect.Type;
 import java.lang.annotation.Annotation;
 
 
@@ -34,52 +36,56 @@ import java.lang.annotation.Annotation;
  * @author Thomas Heller
  */
 @SuppressWarnings("serial")
-public class ConstantParameter
-        implements Parameter, Serializable {
+public class ConstantParameter implements Parameter, Serializable {
 
     private final Object value;
+    
 
     public ConstantParameter(Object value) {
         this.value = value;
     }
 
-    public Object resolveInstance(PicoContainer container,
-                                  ComponentAdapter adapter,
-                                  Class expectedType,
-                                  NameBinding expectedNameBinding,
+    public Object resolveInstance(PicoContainer container, ComponentAdapter<?> adapter,
+                                  Type expectedType, NameBinding expectedNameBinding,
                                   boolean useNames, Annotation binding) {
         return value;
     }
 
-    public boolean isResolvable(PicoContainer container,
-                                ComponentAdapter adapter,
-                                Class expectedType,
-                                NameBinding expectedNameBinding,
-                                boolean useNames, Annotation binding) {
-        try {
-            verify(container, adapter, expectedType, expectedNameBinding, useNames, binding);
-            return true;
-        } catch(final PicoCompositionException e) {
-            return false;
-        }
+    public boolean isResolvable(PicoContainer container, ComponentAdapter<?> adapter, Type expectedType, NameBinding expectedNameBinding, boolean useNames, Annotation binding) {
+        if (expectedType instanceof Class) {
+            return isAssignable((Class) expectedType);
+        } else if (expectedType instanceof ParameterizedType) {
+        	return isAssignable( ((ParameterizedType)expectedType).getRawType());
+        } 
+        return false;
     }
 
     /**
      * {@inheritDoc}
      *
-     * @see Parameter#verify(PicoContainer, ComponentAdapter, Class, NameBinding ,boolean, Annotation)
+     * @see Parameter#verify(org.picocontainer.PicoContainer,org.picocontainer.ComponentAdapter,java.lang.reflect.Type,org.picocontainer.NameBinding,boolean,java.lang.annotation.Annotation)
      */
-    public void verify(PicoContainer container,
-                       ComponentAdapter adapter,
-                       Class expectedType,
-                       NameBinding expectedNameBinding,
+    public void verify(PicoContainer container, ComponentAdapter<?> adapter,
+                       Type expectedType, NameBinding expectedNameBinding,
                        boolean useNames, Annotation binding) throws PicoException {
-        if (!checkPrimitive(expectedType) && !expectedType.isInstance(value)) {
-            throw new PicoCompositionException(expectedType.getClass().getName() + " is not assignable from " +
-                                                 (value != null ? value.getClass().getName() : "null"));
+        if (!isAssignable(expectedType)) {
+            throw new PicoCompositionException(
+                expectedType + " is not assignable from " +
+                        (value != null ? value.getClass().getName() : "null"));
         }
     }
 
+    protected boolean isAssignable(Type expectedType) {
+        boolean isAssignable;
+        if (expectedType instanceof Class) {
+            Class expectedClass = (Class) expectedType;
+            if (checkPrimitive(expectedClass) || expectedClass.isInstance(value)) {
+                return true;
+            }
+        }
+        return false;
+    }
+
     /**
      * Visit the current {@link Parameter}.
      *
